#ifndef MSP_GL_SL_COMPATIBILITY_H_
#define MSP_GL_SL_COMPATIBILITY_H_

#include <string>
#include "visitor.h"

namespace Msp {
namespace GL {
namespace SL {

/** Generates default precision declarations or removes precision declarations
according to the requirements of the target API. */
class PrecisionConverter: private TraversingVisitor
{
private:
	Stage *stage;
	std::set<std::string> have_default;
	NodeList<Statement>::iterator insert_point;
	std::set<Node *> nodes_to_remove;

public:
	PrecisionConverter();

	void apply(Stage &);

private:
	virtual void visit(Block &);
	virtual void visit(Precision &);
	virtual void visit(VariableDeclaration &);
};

/** Converts structures of the syntax tree to match a particular set of
features. */
class LegacyConverter: private TraversingVisitor
{
private:
	Stage *stage;
	Features features;
	VariableDeclaration *frag_out;
	NodeList<Statement>::iterator uniform_insert_point;
	std::set<Node *> nodes_to_remove;

public:
	LegacyConverter();

	virtual void apply(Stage &, const Features &);

private:
	void unsupported(const std::string &);

	virtual void visit(Block &);
	bool check_version(const Version &) const;
	bool check_extension(bool Features::*) const;
	bool supports_stage(Stage::Type) const;
	bool supports_unified_interface_syntax() const;
	virtual void visit(VariableReference &);
	virtual void visit(Assignment &);
	bool supports_unified_sampling_functions() const;
	virtual void visit(FunctionCall &);
	bool supports_interface_layouts() const;
	bool supports_centroid_sampling() const;
	bool supports_sample_sampling() const;
	virtual void visit(VariableDeclaration &);
	bool supports_interface_blocks(const std::string &) const;
	virtual void visit(InterfaceBlock &);
};

} // namespace SL
} // namespace GL
} // namespace Msp

#endif
